/******************************************************************************
*									      *
*	(C) 1993 by K. Ballueder					      *
*									      *
*	See README and COPYING for details.				      *
*									      *
*		kballued@charon.physik.uni-osnabrueck.de		      *
*		kballued@jupiter.rz.uni-osnabrueck.de			      *
*									      *
**** Changelog: ***************************************************************
*/

% GClass: Class GObject: Object{
// components
	int		width;
	int		height;
	int 		bytesperpixel;
	void *		buffer;
%-
	GObject @ setdata(_self, void *data);
	GObject @ godisplay(_self, int x, int y);
	GObject @ goundisplay(_self, int x, int y);
	GObject @ gomove(_self, int x1, int y1, int x2, int y2);
	GObject @ save(_self, FILE *fp);
	GObject @ load(_self, FILE *fp);
%} 

#define width(g)    (((const struct GObject *)(g)) -> width)
#define height(g)    (((const struct GObject *)(g)) -> height)

