/******************************************************************************
*									      *
*	(C) 1993 by K. Ballueder					      *
*									      *
*	See README and COPYING for details.				      *
*									      *
*	Please feel free to contact me:					      *
*		kballued@charon.physik.uni-osnabrueck.de		      *
*		kballued@jupiter.rz.uni-osnabrueck.de			      *
*									      *
******************************************************************************/


/*****

      GAME.C: Routinen zur Punkteverwaltung etc

*****/

extern void tod(void);  /* Tod von Spieler 1! */
#ifdef ENABLE_CHEATING
extern 	int lives;
#endif
